@@ -22,6 +22,7 @@ import os
 import logging
 import io
 
+from . import _
 from . import common
 from . import metadata
 
@@ -45,15 +46,19 @@ def main():
 
     global config, options
 
+    supported = ['txt', 'yml']
+
     # Parse command line...
     parser = ArgumentParser(usage="%(prog)s [options] [APPID [APPID ...]]")
     common.setup_global_opts(parser)
     parser.add_argument("-l", "--list", action="store_true", default=False,
-                        help="List files that would be reformatted")
+                        help=_("List files that would be reformatted"))
     parser.add_argument("-t", "--to", default=None,
-                        help="Rewrite to a specific format")
-    parser.add_argument("appid", nargs='*', help="app-id in the form APPID")
+                        help=_("Rewrite to a specific format: ") + ', '.join(supported))
+    parser.add_argument("appid", nargs='*', help=_("applicationId in the form APPID"))
+    metadata.add_metadata_arguments(parser)
     options = parser.parse_args()
+    metadata.warnings_action = options.W
 
     config = common.read_config(options)
 
@@ -62,18 +67,22 @@ def main():
     apps = common.read_app_args(options.appid, allapps, False)
 
     if options.list and options.to is not None:
-        parser.error("Cannot use --list and --to at the same time")
-
-    supported = ['txt', 'yml']
+        parser.error(_("Cannot use --list and --to at the same time"))
 
     if options.to is not None and options.to not in supported:
-        parser.error("Must give a valid format to --to")
+        parser.error(_("Unsupported metadata format, use: --to [{supported}]")
+                     .format(supported=' '.join(supported)))
 
     for appid, app in apps.items():
-        base, ext = common.get_extension(app.metadatapath)
+        path = app.metadatapath
+        base, ext = common.get_extension(path)
         if not options.to and ext not in supported:
-            logging.info("Ignoring %s file at '%s'" % (ext, app.metadatapath))
+            logging.info(_("Ignoring {ext} file at '{path}'").format(ext=ext, path=path))
             continue
+        elif options.to is not None:
+            logging.info(_("Rewriting '{appid}' to '{path}'").format(appid=appid, path=options.to))
+        else:
+            logging.info(_("Rewriting '{appid}'").format(appid=appid))
 
         to_ext = ext
         if options.to is not None:
@@ -81,15 +90,27 @@ def main():
 
         if options.list:
             if not proper_format(app):
-                print(app.metadatapath)
+                print(path)
             continue
 
+        newbuilds = []
+        for build in app.builds:
+            new = metadata.Build()
+            for k in metadata.build_flags:
+                v = build[k]
+                if v is None or v is False or v == [] or v == '':
+                    continue
+                new[k] = v
+            newbuilds.append(new)
+        app.builds = newbuilds
+
         metadata.write_metadata(base + '.' + to_ext, app)
 
         if ext != to_ext:
-            os.remove(app.metadatapath)
+            os.remove(path)
+
+    logging.debug(_("Finished"))
 
-    logging.debug("Finished.")
 
 if __name__ == "__main__":
     main()
