@@ -19,12 +19,14 @@
 import sys
 import os
 import glob
+import requests
 from argparse import ArgumentParser
 import logging
 
+from . import _
 from . import common
 from . import net
-from .common import FDroidException
+from .exception import FDroidException
 
 options = None
 config = None
@@ -37,19 +39,19 @@ def main():
     # Parse command line...
     parser = ArgumentParser(usage="%(prog)s [options] [APPID[:VERCODE] [APPID[:VERCODE] ...]]")
     common.setup_global_opts(parser)
-    parser.add_argument("appid", nargs='*', help="app-id with optional versioncode in the form APPID[:VERCODE]")
+    parser.add_argument("appid", nargs='*', help=_("applicationId with optional versionCode in the form APPID[:VERCODE]"))
     options = parser.parse_args()
 
     config = common.read_config(options)
 
     tmp_dir = 'tmp'
     if not os.path.isdir(tmp_dir):
-        logging.info("Creating temporary directory")
+        logging.info(_("Creating temporary directory"))
         os.makedirs(tmp_dir)
 
     unsigned_dir = 'unsigned'
     if not os.path.isdir(unsigned_dir):
-        logging.error("No unsigned directory - nothing to do")
+        logging.error(_("No unsigned directory - nothing to do"))
         sys.exit(0)
 
     verified = 0
@@ -60,7 +62,7 @@ def main():
     for apkfile in sorted(glob.glob(os.path.join(unsigned_dir, '*.apk'))):
 
         apkfilename = os.path.basename(apkfile)
-        appid, vercode = common.apknameinfo(apkfile)
+        appid, vercode = common.publishednameinfo(apkfile)
 
         if vercodes and appid not in vercodes:
             continue
@@ -69,18 +71,25 @@ def main():
 
         try:
 
-            logging.info("Processing " + apkfilename)
+            logging.info("Processing {apkfilename}".format(apkfilename=apkfilename))
 
             remoteapk = os.path.join(tmp_dir, apkfilename)
             if os.path.exists(remoteapk):
                 os.remove(remoteapk)
             url = 'https://f-droid.org/repo/' + apkfilename
             logging.info("...retrieving " + url)
-            net.download_file(url, dldir=tmp_dir)
+            try:
+                net.download_file(url, dldir=tmp_dir)
+            except requests.exceptions.HTTPError as e:
+                try:
+                    net.download_file(url.replace('/repo', '/archive'), dldir=tmp_dir)
+                except requests.exceptions.HTTPError as e:
+                    raise FDroidException(_('Downloading {url} failed. {error}')
+                                          .format(url=url, error=e))
 
-            compare_result = common.compare_apks(
-                os.path.join(unsigned_dir, apkfilename),
+            compare_result = common.verify_apks(
                 remoteapk,
+                os.path.join(unsigned_dir, apkfilename),
                 tmp_dir)
             if compare_result:
                 raise FDroidException(compare_result)
@@ -92,9 +101,10 @@ def main():
             logging.info("...NOT verified - {0}".format(e))
             notverified += 1
 
-    logging.info("Finished")
+    logging.info(_("Finished"))
     logging.info("{0} successfully verified".format(verified))
     logging.info("{0} NOT verified".format(notverified))
 
+
 if __name__ == "__main__":
     main()
